@@ -1,16 +1,20 @@
 from typing import override
+from ccxt.base import precise
 import pandas as pd
+from decimal import Decimal
+
 
 from taker.TrailingSLTaker import TrailingSLTaker
 
 class SMCSLAndTPTaker(TrailingSLTaker):
-    def __init__(self,g_config, platform_config, feishu_webhook=None, monitor_interval=4,logger=None):
-        super().__init__(g_config, platform_config, feishu_webhook, monitor_interval,logger)
+    def __init__(self,g_config, platform_config, common_config=None, feishu_webhook=None, monitor_interval=4,logger=None):
+        super().__init__(g_config, platform_config, common_config, feishu_webhook, monitor_interval,logger)
         self.global_symbol_take_profit_flag = {} # 记录每个symbol是否设置全局止盈标志
         self.global_symbol_take_profit_price = {} # 记录每个symbol的止盈价格
-        self.htf_liquidities = {}
-        self.all_TP_SL_ratio = float(platform_config.get("all_TP_SL_ratio",1.5)) #The profit-loss ratio 盈亏比
-        self.open_trail_profit = bool(platform_config.get("open_trail_profit",True)) # 开仓是否设置止盈
+        self.htf_liquidities_TP = {}
+        self.htf_liquidities_SL = {}
+        # self.all_TP_SL_ratio = float(platform_config.get("all_TP_SL_ratio",1.5)) #The profit-loss ratio 盈亏比
+        # self.open_trail_profit = bool(platform_config.get("open_trail_profit",True)) # 开仓是否设置止盈
        
     @override  
     def check_reverse_position(self,symbol,position,pair_config):
@@ -42,8 +46,13 @@ class SMCSLAndTPTaker(TrailingSLTaker):
 
         for index in range(1, len(data)):
       
-            data.at[index, 'Up'] = max(data.at[index - 1, 'Up'], data.at[index, 'high'])
-            data.at[index, 'Dn'] = min(data.at[index - 1, 'Dn'], data.at[index, 'low'])
+            prev_up = self.toDecimal(data.at[index - 1, 'Up'])
+            curr_high = self.toDecimal(data.at[index, 'high'])
+            prev_dn = self.toDecimal(data.at[index - 1, 'Dn'])
+            curr_low = self.toDecimal(data.at[index, 'low'])
+            
+            data.at[index, 'Up'] = max(prev_up, curr_high)
+            data.at[index, 'Dn'] = min(prev_dn, curr_low)
             data.at[index, 'pos'] = data.at[index - 1, 'pos']
             data.at[index, 'iUp'] = data.at[max(0,index - 1), 'iUp'] if data.at[max(0,index - 1), 'iUp'] is not None else index
             data.at[index, 'iDn'] = data.at[max(0,index - 1), 'iDn'] if data.at[max(0,index - 1), 'iDn'] is not None else index
@@ -53,10 +62,10 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             pvtLo = self.is_pivot_low(data, index, prd, check_bounds)
 
             if pvtHi:
-                data.at[index, 'Up'] = data.at[index, 'high']
+                data.at[index, 'Up'] = self.toDecimal(data.at[index, 'high'])
                 data.at[index, 'iUp'] = index
             if pvtLo:
-                data.at[index, 'Dn'] = data.at[index, 'low']
+                data.at[index, 'Dn'] = self.toDecimal(data.at[index, 'low'])
                 data.at[index, 'iDn'] = index
             # 寻找Bullish结构
             if data.at[index, 'Up'] > data.at[index - 1, 'Up']:
@@ -80,7 +89,7 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         
             # # 寻找Bearish结构
             if data.at[index, 'Dn'] < data.at[index - 1, 'Dn']:
-                data.at[index, 'iDn'] = index # TODO
+                data.at[index, 'iDn'] = index 
                 if data.at[index - 1, 'pos'] >= 0:
                 
                     data.at[index, 'pattern'] = 'Bearish_CHoCH'
@@ -174,8 +183,8 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             return False
         current_high = data.at[index, 'high']
         prev_highs = data['high'].iloc[max(0,index - period):index]
-        next_highs = data['high'].iloc[index :min(len(data),index + period + 1)]
-        return all(current_high >= prev_highs) and all(current_high >= next_highs)
+        next_highs = data['high'].iloc[index+1 :min(len(data),index + period )+1]
+        return all(current_high >= prev_highs) and all(current_high > next_highs)
 
     def is_pivot_low(self, data, index, period, check_bounds=False):
         """
@@ -189,14 +198,14 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             return False
         current_low = data.at[index, 'low']
         prev_lows = data['low'].iloc[max(0,index - period):index]
-        next_lows = data['low'].iloc[index :min(len(data),index + period + 1)]
-        return all(current_low <= prev_lows) and all(current_low <= next_lows)
+        next_lows = data['low'].iloc[index+1 :min(len(data),index + period)+1]
+        return all(current_low <= prev_lows) and all(current_low < next_lows)
 
               
     def set_sl_by_profit(self, symbol, position, profit, pair_config, kLines=None):
         
         """
-        根据利润设置止损价
+        根据扫过的流动性移动止损
         Args:
             symbol: 交易对
             position: 仓位信息
@@ -205,75 +214,53 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             kLines: K线数据
         """
 
-        total_profit = profit      
 
-        current_tier = '无'
-        # 各档止盈逻辑
-        # 确定当前盈利档位
-        if self.open_trail_profit and total_profit >= self.second_trail_profit_threshold:
-            current_tier = "高档"
-     
-        elif self.open_trail_profit and total_profit>= self.first_trail_profit_threshold:
-            current_tier = "中档"
-         
-        elif self.open_trail_profit and total_profit >= self.low_trail_profit_threshold:
-            current_tier = "低档"
-         
-        # 根据不同档位设置止损价格,没有单独为交易对设置，用全局参数代替
-        tier_config = {
-            "低档": {
-                "stop_loss_pct": float(pair_config.get('low_trail_stop_loss_pct',self.low_trail_stop_loss_pct)),
-                         
-            },
-            "中档": {
-                "stop_loss_pct": float(pair_config.get('trail_stop_loss_pct',self.trail_stop_loss_pct)),
+        
+        if self.open_trail_profit and self.global_symbol_stop_loss_flag.get(symbol,False):
             
-            },
-            "高档": {
-                "stop_loss_pct": float(pair_config.get('higher_trail_stop_loss_pct',self.higher_trail_stop_loss_pct)),
+            smc_strategy = pair_config.get('smc_strategy',{})
+            htf = str(smc_strategy.get('HTF','15m')) 
+            htf_prd = int(smc_strategy.get('HTF_swing_points_length',3))           
+            ltf = str(smc_strategy.get('LTF','1m'))
+            ltf_prd = int(smc_strategy.get('LTF_swing_points_length',3))
+           
+            
+            ctf = ltf
+            ctf_prd = ltf_prd
+            # ctf_Klines = kLines
+            
+            # 寻找LTF的流动性，作为移动止损位置
+            ctf_Klines = self.get_historical_klines(symbol=symbol, bar=ctf)
+            ctf_df = self.format_klines(ctf_Klines)
+            ctf_df_with_struct = self.build_struct(df=ctf_df,prd=ctf_prd)
+            # self.logger.debug(f"{symbol} : {ctf} ctf_df_with_struct\n{ctf_df_with_struct}")
 
-            }
-        }
-        self.logger.info(
-                f"{symbol} 档位[{current_tier}]: 盈利: {total_profit:.2f}%")
+            marketPrice = self.toDecimal(position['markPrice'])
 
-        if current_tier in tier_config:
-            config = tier_config[current_tier]  
 
-                 
-
-            # 计算回撤止损价格
-            sl_price = self.calculate_stop_loss_price(
-                symbol=symbol, 
-                position=position,
-                stop_loss_pct=config['stop_loss_pct']
-            )
+            # 寻找流动性
+            htf_liquidity_SL = self.htf_liquidities_SL.get(symbol,None)
+            if htf_liquidity_SL is None:
+                htf_liquidity_SL = self.detect_liquidity_for_SL(symbol, ctf_df_with_struct, position['side'], marketPrice)
+                self.htf_liquidities_SL[symbol] = htf_liquidity_SL
+            
+            if len(htf_liquidity_SL) == 0:
+                self.logger.info(f"{symbol} : {ctf} 没有找到扫荡的流动性，不重设止损")
+                return
+        
+            sl_price = self.calculate_trailing_sl_price_by_liquidity(symbol, position, htf_liquidity_SL, self.stop_loss_pct)
+       
             
             # 检查价格是否变化
-            latest_sl_price = self.exchange.safe_float(self.global_symbol_stop_loss_price, symbol, 0.0)
+            latest_sl_price = self.global_symbol_stop_loss_price.get(symbol,0.0)
+            side = position['side']
             if sl_price == latest_sl_price:
-                self.logger.debug(f"{symbol}: 回撤止损价格{latest_sl_price}未变化，不设置")
+                self.logger.debug(f"{symbol}: 移动止损价格{latest_sl_price}未变化，不设置")
                 return
-            
-            # 根据持仓方向判断是否需要更新止损价格
-            should_update = False
-            if position['side'] == 'long':
-                # 多仓时,新止损价需高于当前止损价
-                should_update = sl_price >= latest_sl_price
-                if not should_update:
-                    self.logger.debug(f"{symbol}: 多仓回撤止损价格{sl_price}低于当前止损价格{latest_sl_price}，不更新")
-                else:
-                    self.logger.debug(f"{symbol}: 多仓回撤止损价格{sl_price}高于当前止损价格{latest_sl_price}，更新")
-            else:
-                # 空仓时,新止损价需低于当前止损价
-                should_update = sl_price <= latest_sl_price
-                if not should_update:
-                    self.logger.debug(f"{symbol}: 空仓回撤止损价格{sl_price}高于当前止损价格{latest_sl_price}，不更新")
-                else:
-                    self.logger.debug(f"{symbol}: 空仓回撤止损价格{sl_price}低于当前止损价格{latest_sl_price}，更新")
-            
-            if not should_update:
-                return  
+            elif (side == 'long' and sl_price < latest_sl_price) or (side == 'short' and sl_price > latest_sl_price):
+                self.logger.debug(f"{symbol}: [{side}] 新移动止损价格{sl_price} 不能替换 {latest_sl_price}，不设置")                
+                return
+                
             
             self.cancel_all_algo_orders(symbol=symbol, attachType='SL')
                 
@@ -285,12 +272,13 @@ class SMCSLAndTPTaker(TrailingSLTaker):
          
                 self.global_symbol_stop_loss_price[symbol] = sl_price
                 self.global_symbol_stop_loss_flag[symbol] = True
-                cur_highest_total_profit = self.highest_total_profit.get(symbol, 0.0) 
+                # cur_highest_total_profit = self.highest_total_profit.get(symbol, 0.0) 
                 
                 # 发送通知
-                msg = (f"{symbol}: 盈利达到【{current_tier}】阈值，最高总盈利: {cur_highest_total_profit:.2f}%,"
-                      f"当前盈利回撤到: {total_profit:.2f}%，市场价格:{position['markPrice']},"
-                      f"设置回撤止损位: {sl_price:.9f}")
+                # msg = (f"{symbol}: 盈利达到【{current_tier}】阈值，最高总盈利: {cur_highest_total_profit:.2f}%,"
+                #       f"当前盈利回撤到: {total_profit:.2f}%，市场价格:{position['markPrice']},"
+                #       f"设置回撤止损位: {sl_price:.9f}")
+                msg = (f"{symbol}: {ctf}移动止损，找到扫荡的流动性位置，设置止损: {sl_price:.9f}")
                 self.logger.info(msg)
                 self.send_feishu_notification(msg)
                 
@@ -299,7 +287,7 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             # 默认全局止损            
             if_success = self.set_global_stop_loss(symbol=symbol, position=position, pair_config=pair_config ,kLines = kLines)
             if if_success:
-                # 更新回撤止损价格 
+                # 更新回撤止损价格
                 self.global_symbol_take_profit_flag[symbol] = False
         
     def find_liquidity(self, symbol, data, liquidity_type="BSL") -> pd.DataFrame:
@@ -318,7 +306,7 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         
         return df[df.index == df[col_prefix]].sort_index(ascending=False)
     
-    def detect_liquidity_for_TP(self, symbol, data, side , market_price) -> pd.DataFrame:
+    def detect_liquidity_for_TP(self, symbol, data, side , market_price: Decimal) -> pd.DataFrame:
         """
         TP校对流动性，用市场价格校验流动性是否有效,做多则流动性在市场价格之上，做空流动性要在市场价格之下。
         Args:
@@ -358,7 +346,7 @@ class SMCSLAndTPTaker(TrailingSLTaker):
                     
         return df_valid_liquidities.loc[result_indices].sort_index(ascending=False)
     
-    def detect_liquidity_for_SL(self, symbol, data, side, market_price) -> pd.DataFrame:
+    def detect_liquidity_for_SL(self, symbol, data, side, market_price:Decimal) -> pd.DataFrame:
         """
         SL校对流动性，用市场价格校验流动性是否有效,做多则流动性在市场价格之下，做空流动性要在市场价格之上。
         Args:
@@ -371,54 +359,104 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         is_buy = side == 'long'
         col_prefix = 'iDn' if is_buy else 'iUp'
         price_col = 'Dn' if is_buy else 'Up'
-        # 设置SL,long寻找BSL,short寻找SSL
-        liquidity_type = 'BSL' if is_buy else 'SSL'
+        df = data.copy()  
         
-        df_liquidities = self.find_liquidity(symbol, data, liquidity_type=liquidity_type)
 
-        df_valid_liquidities = df_liquidities.copy()     
-        df_valid_liquidities['was_swept'] = None  
-        df_valid_liquidities['took_out'] = None   
+        # 初始化新列
+
+        df['pivot_price'] = pd.NA
+        df['is_extreme'] = False
+
+        # 1. 根据pos值分组（连续相同pos为一个区间）
+
+        groups = df['pos'].gt(0).ne(df['pos'].gt(0).shift()).cumsum()
+
+        # 2. 对每个pos区间处理
+        for group_id, group_data in df.groupby(groups):
+            pos_val = group_data['pos'].iloc[0]
+            
+            if pos_val > 0:
+                # 找high最大值及其第一次出现的位置
+                max_high = group_data['high'].max()
+                max_idx = group_data['high'].idxmax()
+                df.at[max_idx, 'pivot_price'] = max_high
+                df.at[max_idx, 'is_extreme'] = True
+                
+            elif pos_val < 0:
+                # 找low最小值及其第一次出现的位置
+                min_low = group_data['low'].min()
+                min_idx = group_data['low'].idxmin()
+                df.at[min_idx, 'pivot_price'] = min_low
+                df.at[min_idx, 'is_extreme'] = True
+
+
+        # self.logger.debug(f"{df[['timestamp', 'pattern','high','low','pivot_price','is_extreme']]}")
         
+        struct_siginfo = "Bullish" if is_buy else "Bearish"
+
+        # 3. 筛选CHoCH后符合条件的行
         result_indices = []
-        current_price = float('inf') if is_buy else float('-inf')
-        current_i = float('inf') 
-        took_out_status = False
-        
-        # 遍历并筛选符合条件的记录，找到扫过流动性的
-        for idx, row in df_valid_liquidities.iterrows():
-            if not is_buy:
-                if row[price_col] > market_price:
-                    if  row[price_col] > current_price and row[col_prefix] < current_i:
-                        result_indices.append(idx)
-                        
-                        if current_i != float('inf')  and  not df_valid_liquidities.at[current_i, 'took_out'] :
-                            df_valid_liquidities.at[idx, 'took_out'] = True
-                            
-                        current_price = row[price_col]
-                        current_i = row[col_prefix]
-                        
-                    else:
-                        df_valid_liquidities.at[idx, 'was_swept'] = True
-               
-            else:
-                if row[price_col] < market_price:
-                    if  row[price_col] < current_price and row[col_prefix] < current_i:
-                        result_indices.append(idx)
-                        
-                        if current_i != float('inf')  and  not df_valid_liquidities.at[current_i, 'took_out'] :
-                            df_valid_liquidities.at[idx, 'took_out'] = True
-                        current_price = row[price_col]
-                        current_i = row[col_prefix]
-                    else:
-                        df_valid_liquidities.at[idx, 'was_swept'] = True
-                        
-        # 过滤出took_out为True的记录
-        took_out_df = df_valid_liquidities.loc[result_indices]
-        took_out_df = took_out_df[took_out_df['took_out'] == True]
+        # 倒序遍历数据
+        found_choch = False
+        for idx in reversed(df.index):
+           
+            # 检查是否有新的pattern
+            current_pattern = df.at[idx, 'pattern']
+            
+            
+            # 如果找到对应的CHoCH结构
+            if current_pattern and f"{struct_siginfo}_CHoCH" == current_pattern:
+                found_choch = True
+                continue
+   
+            # 在找到CHoCH结构后,记录极值点
+            if found_choch and df.at[idx, 'is_extreme']:
+                result_indices.append(idx)
+                found_choch = False
+                
+        self.logger.debug(f"result_indices = {result_indices}")
+        took_out_df = df.loc[result_indices].copy()
+        valid_mast = took_out_df['pivot_price'] < market_price if is_buy else took_out_df['pivot_price'] > market_price
+        took_out_df = took_out_df[valid_mast]
         return took_out_df.sort_index(ascending=False)
+    
+   
+    def calculate_trailing_sl_price_by_liquidity(self, symbol, position, df_liquidities, stop_loss_pct=2, offset=1) -> Decimal:
+        """
+        计算回撤止损价格，根据流动性，做多则回撤止损价格在流动性之下，做空则回撤止损价格在流动性之上。
+        Args:
+            symbol (str): 交易对
+            position (dict): 仓位信息
+            df_liquidities (pd.DataFrame): 流动性数据
+            stop_loss_pct (int, optional): 回撤百分比. Defaults to 2.
+            offset (int, optional): 偏移量. Defaults to 1.
+        Returns:
+            Decimal: 回撤止损价格
+        """
+        sl_price =  self.calculate_sl_price_by_pct(symbol, position, stop_loss_pct)
+        precision = self.get_precision_length(symbol)
+
+               
+        is_buy = position['side'] == 'long'
+        price_col = 'Dn' if is_buy else 'Up'
+        self.logger.debug(f"{symbol} : SL side={position['side']} sl_price={sl_price:.{precision}} \n SL的扫荡流动性=\n {df_liquidities[['timestamp',price_col, 'is_extreme']]}")
         
-    def calculate_tp_price_by_liquidity(self, symbol, position, df_liquidities, stop_loss_pct=2, tp_sl_ratio=1.5, offset=1) -> float:
+        # valid_mask = df_liquidities[price_col] > sl_price if is_buy else df_liquidities[price_col] < sl_price
+        # df_valid_liquidities = df_liquidities[valid_mask]
+        # 获取止损价格
+        # trailing_sl = df_valid_liquidities.iloc[0][price_col] if len(df_valid_liquidities) > 0 else sl_price
+        trailing_sl = self.toDecimal(df_liquidities[price_col].max() if is_buy else df_liquidities[price_col].min())
+                
+        # 计算移动止损价格 , 做多则止损价格在流动性之下tick_size，做空则止损价格在流动性之上tick_size。
+        tick_size = self.get_tick_size(symbol)
+        if is_buy:
+            trailing_sl = trailing_sl - offset * tick_size
+        else:   
+            trailing_sl = trailing_sl + offset * tick_size
+            
+        return self.round_price_to_tick(symbol, trailing_sl)
+    
+    def calculate_tp_price_by_liquidity(self, symbol, position, df_liquidities, stop_loss_pct=2, tp_sl_ratio=1.5, offset=1) -> Decimal:
         """_summary_
         计算止盈价格，根据流动性，做多则止盈价格在流动性之上，做空则止盈价格在流动性之下。
         Args:
@@ -430,35 +468,45 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             offset (int, optional): _description_. Defaults to 1.
 
         Returns:
-            float: _description_
+            Decimal: _description_
         """        
         
-        tp_price = 0.0
+        tp_price = Decimal('0.0')
         # market_price = float(position['markPrice'])
         
         is_buy = position['side'] == 'long'
         price_col = 'Up' if is_buy else 'Dn'
         
         # sl_price = self.global_symbol_stop_loss_price.get(symbol, float(position['markPrice']))
-        entry_price = float(position['entryPrice'])
-        sl_price =  self.calculate_sl_price_by_pct(symbol, position, stop_loss_pct)
-        threshold = 0.0
-        if is_buy and sl_price > 0 :
-            threshold = float(self.round_price_to_tick(symbol, (entry_price - sl_price ) * tp_sl_ratio + entry_price))
-        elif not is_buy and sl_price > 0:
-            threshold = float(self.round_price_to_tick(symbol, entry_price - (sl_price - entry_price ) * tp_sl_ratio))
-         
+        # 获取开仓价格和止损价格
+        entry_price = self.toDecimal(position['entryPrice'])
+        sl_price = self.calculate_sl_price_by_pct(symbol, position, stop_loss_pct)
+        
+        # 计算止盈阈值
+        threshold = Decimal('0.0')
+        if sl_price <= 0:
+            return threshold
+            
+     
+        # 计算开仓价格和止损价格的差值
+        price_diff = abs(entry_price - sl_price)
+        
+        # 根据方向计算止盈阈值
+        target_price = entry_price + price_diff * self.toDecimal(tp_sl_ratio) if is_buy else entry_price - price_diff * self.toDecimal(tp_sl_ratio)
+        threshold = self.round_price_to_tick(symbol, target_price)
 
-        # 过滤有效的流动性价格
+        precision = self.get_precision_length(symbol)
+         
+        # 根据方向过滤有效的流动性价格
+        df_liquidities[price_col] = df_liquidities[price_col].apply(self.toDecimal)
         valid_mask = df_liquidities[price_col] > threshold if is_buy else df_liquidities[price_col] < threshold
         df_valid_liquidities = df_liquidities[valid_mask]
-        self.logger.debug(f"{symbol} : threshold={threshold} sl_price={sl_price} 有效的流动=\n {df_valid_liquidities[['timestamp','Up','Dn']]}")
-        # 获取止盈价格
+        
+        self.logger.debug(f"{symbol} : TP threshold={threshold:.{precision}} sl_price={sl_price:.{precision}} 有效的流动=\n {df_valid_liquidities[['timestamp','Up','Dn']]}")
+        
+        # 获取止盈价格并确保其满足方向要求
         tp_price = df_valid_liquidities.iloc[0][price_col] if len(df_valid_liquidities) > 0 else threshold
-        if is_buy:
-            tp_price = max(tp_price, threshold)
-        else:
-            tp_price = min(tp_price, threshold)
+        tp_price = self.toDecimal(max(tp_price, threshold) if is_buy else min(tp_price, threshold))
         tick_size = self.get_tick_size(symbol)
         
         # 计算止盈价格 , 做多则止盈价格在流动性之下tick_size，做空则止盈价格在流动性之上tick_size。
@@ -468,12 +516,13 @@ class SMCSLAndTPTaker(TrailingSLTaker):
             tp_price = tp_price + offset * tick_size
             
         
-        return tp_price
+        return self.round_price_to_tick(symbol, tp_price)
     
     @override
     def close_all_cache(self):
         super().close_all_cache()
-        self.htf_liquidities.clear()
+        self.htf_liquidities_TP.clear()
+        self.htf_liquidities_SL.clear()
         self.global_symbol_take_profit_flag.clear()
         self.global_symbol_take_profit_price.clear()
 
@@ -481,7 +530,8 @@ class SMCSLAndTPTaker(TrailingSLTaker):
     @override
     def reset_all_cache(self, symbol):
         super().reset_all_cache(symbol)
-        self.htf_liquidities[symbol] = None
+        self.htf_liquidities_TP[symbol] = None
+        self.htf_liquidities_SL[symbol] = None
         self.global_symbol_take_profit_flag[symbol] = False
         self.global_symbol_take_profit_price[symbol] = None
            
@@ -489,9 +539,12 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         """
         根据结构设置止盈
         """
+
+        presision = self.get_precision_length(symbol)
+       
         # 如果已经触发过全局止盈，则跳过 
         if self.global_symbol_take_profit_flag.get(symbol, False):            
-            self.logger.info(f"{symbol} : 已经设置过全局止盈 tp_price={self.global_symbol_take_profit_price[symbol]}")
+            self.logger.info(f"{symbol} : 已经设置过全局止盈 tp_price={self.global_symbol_take_profit_price[symbol]:.{presision}}")
             return
 
         smc_strategy = pair_config.get('smc_strategy',{})
@@ -505,10 +558,10 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         htf_df_with_struct = self.build_struct(df=htf_df,prd=htf_prd)  
 
         # 寻找流动性
-        htf_liquidity = self.htf_liquidities.get(symbol,None)
+        htf_liquidity = self.htf_liquidities_TP.get(symbol,None)
         if htf_liquidity is None:
-            htf_liquidity = self.detect_liquidity_for_TP(symbol, htf_df_with_struct, position['side'], position['markPrice'])
-            self.htf_liquidities[symbol] = htf_liquidity
+            htf_liquidity = self.detect_liquidity_for_TP(symbol, htf_df_with_struct, position['side'], self.toDecimal(position['markPrice']))
+            self.htf_liquidities_TP[symbol] = htf_liquidity
         
         if len(htf_liquidity) <= 0:
             self.logger.info(f"{symbol} : 没有找到流动性，不设置止盈")
@@ -521,7 +574,7 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         if self.set_take_profit(symbol, position, tp_price):
             self.global_symbol_take_profit_flag[symbol] = True
             self.global_symbol_take_profit_price[symbol] = tp_price
-            self.logger.info(f"{symbol} : [{position['side']}] 设置全局止盈价={tp_price}")
+            self.logger.info(f"{symbol} : [{position['side']}] 设置全局止盈价={tp_price:.{presision}}")
     
  
     def check_total_profit(self, symbol, position):
@@ -533,8 +586,11 @@ class SMCSLAndTPTaker(TrailingSLTaker):
         cur_highest_total_profit = self.highest_total_profit.get(symbol, 0.0)    
         if total_profit > cur_highest_total_profit:            
             self.highest_total_profit[symbol] = total_profit
-        profit_key = "盈利" if total_profit > 0.0 else "亏损"
-        msg = f"{symbol} : {profit_key}={total_profit:.2f}% 方向={position['side']} 开仓={position['entryPrice']:.6f} 市价={position['markPrice']:.6f}"
+        
+        precision = self.get_precision_length(symbol)
+        entryPrice = self.toDecimal(position['entryPrice'])
+        marketPrice = self.toDecimal(position['markPrice'])
+        msg = f"{symbol} : 盈利={total_profit:.2f}% 方向={position['side']} 开仓={entryPrice:.{precision}f} 市价={marketPrice:.{precision}f}"
         self.logger.info(msg)
         self.send_feishu_notification(msg)    
         
