@@ -73,6 +73,7 @@ class StrategyTaker():
     PD_LOW_COL = SMCPDArray.SMCPDArray.PD_LOW_COL
     PD_MID_COL = SMCPDArray.SMCPDArray.PD_MID_COL 
     PD_TYPE_COL = SMCPDArray.SMCPDArray.PD_TYPE_COL
+    PD_WAS_BALANCED_COL = SMCPDArray.SMCPDArray.PD_WAS_BALANCED_COL
     
     def __init__(self, g_config,  platform_config, common_config, logger=None ,exchangeKey='okx'):
         """_summary_
@@ -423,19 +424,21 @@ class StrategyTaker():
 
         return self.smcFVG.find_FVGs(data, side, check_balanced, start_index)
 
-    def build_struct(self, symbol, data) -> pd.DataFrame:
+    def build_struct(self, symbol, data, tf=None,  is_struct_body_break=True) -> pd.DataFrame:
         
         """_summary_
             构建SMC结构，参考 Tradingview OP@SMC Structures and FVG
         Args:
             symbol (_type_): _description_
+            tf (_type_): _description_
             data (_type_): _description_
+            is_struct_body_break (bool, optional): _description_. Defaults to True.
         Returns:
             _type_: _description_    
         """
         
                 
-        return self.smcStruct.build_struct(data)
+        return self.smcStruct.build_struct(data, tf=None, is_struct_body_break=is_struct_body_break)
         
     def get_latest_struct(self, symbol, data) -> dict:
         """_summary_
@@ -627,7 +630,8 @@ class StrategyTaker():
         precision = self.get_precision_length(symbol)
         entryPrice = self.toDecimal(position[self.ENTRY_PRICE_KEY])
         marketPrice = self.toDecimal(position[self.MARK_PRICE_KEY])
-        msg = f"{symbol} : 盈利={total_profit:.2f}% 方向={position[self.SIDE_KEY]} 开仓={entryPrice:.{precision}f} 市价={marketPrice:.{precision}f}"
+        profit_msg = "盈利" if total_profit >=0 else "亏损"
+        msg = f"{symbol} : {profit_msg}={total_profit:.2f}% 方向={position[self.SIDE_KEY]} 开仓={entryPrice:.{precision}f} 市价={marketPrice:.{precision}f}"
         self.logger.info(msg)
         self.send_feishu_notification(symbol, msg)  
         
@@ -647,6 +651,13 @@ class StrategyTaker():
           
     def monitor_total_profit(self):
         self.logger.info("启动主循环，开始监控总盈利...")
+     
+        htf = str(self.strategy_config.get('HTF','4h'))  
+        atf = str(self.strategy_config.get('ATF','1h')) 
+        etf = str(self.strategy_config.get('ETF','15m'))
+
+
+        self.logger.info(f"策略时间框架 {htf}|{atf}|{etf} ...\n")
 
         while True:
             
@@ -661,7 +672,7 @@ class StrategyTaker():
         
             except Exception as e:
                 error_message = f"!!获取持仓失败: {str(e)}"
-                self.logger.error(error_message)
+                self.logger.warning(error_message)
                 self.send_feishu_notification("ALL",error_message)               
                 continue
             
