@@ -5,7 +5,9 @@ import logging
 import requests
 from logging.handlers import TimedRotatingFileHandler
 
-
+'''
+三线策略 废弃
+'''
 class ThreeLineTradingTaker:
     def __init__(self, config, feishu_webhook=None, monitor_interval=60):
         self.stop_loss_pct = config["all_stop_loss_pct"]  # 全局止损百分比
